#include  "bitmap.h"
#include  <vector>
#include  <iostream>
#include  <iterator>
#include  <algorithm>


typedef atlas::bitmap::BitMap value_set;
typedef std::vector<value_set> table;

class occurrences
{
  table singles;
  table multiples;

public:
  occurrences (unsigned int max_terms, unsigned long max_sum);

  
  void add(unsigned long term);
  
  const value_set& unique_sums(unsigned int num_terms) const
    { return singles[num_terms]; }

}; 

occurrences::occurrences (unsigned int max_terms, unsigned long max_sum)
: singles(max_terms+1,value_set(max_sum+1))
, multiples(max_terms+1,value_set(max_sum+1))
{ singles[0].insert(0); } 

void occurrences::add(unsigned long term)
{ value_set t (singles[0].capacity());
  value_set d=t; // two temporaries
  for (unsigned i=singles.size(); i-->1; )
  {
    t = singles[i-1];
    t<<=term;
    d=t; // newly created unique(?) expressions
    d &= singles[i]; // double terms, from two unique expressions
    singles[i]^= t; // leave only true unique expressions at |i|
    t = multiples[i-1]; t<<=term; // do same for multiple terms;
    singles[i].andnot((multiples[i] |= t)|=d); // mark and cancel new multiples
  }
}

int main()
{ const unsigned int k=50;
  const unsigned long last_squared=100;
  unsigned long max = 0;
  for (unsigned long i=1; i<=last_squared; ++i)
    max += i*i;

  std::cout << "Computing unique sums of " << k <<  " out of the first "
            << last_squared << " positive squares." << std::endl;

  occurrences state(k,max);

  std::cout << max << ", " << state.unique_sums(1).capacity() << std::endl;
  for (unsigned long i=1; i<=last_squared; ++i)
  { std::cout << "Incorporating square of " << i 
              << " which is " << i*i << std::endl; // show progress
    state.add(i*i); // include terms
  }
  const value_set result = state.unique_sums(k);
  std::cout << "Found " << result.size() << " unique sums:" << std::endl;
  std::ostream_iterator<unsigned long> lister(std::cout,",");
  std::copy (result.begin(), result.end(), lister);
  std::cout << std::endl;
  auto sum = std::accumulate (result.begin(), result.end(), 0);
  std::cout << "Sum is " << sum << '.' << std::endl;
}


